// --------------------------------------------------------------------------------------
//
// AmpelMain.CPP                                                   Ulrich Mller 11.06.02
//                                                                 PCS1700   C++Builder 4
// Betrieb einer Fugngerampel in zwei eigenen Threads :
// frmThreadMain : MainThread Steuerung des Betriebsablaufs, Bedieneroberflche
// AmpelAuto     : Thread     Steuerung der Autoampel
// AmpelFuss     : Thread     Steuerung der Fugngerampel
// umFish30.DLL  : Thread     Zugriff auf das Interface, durch MultiMediaTimer gesteuert
//
// Modell : Intelligent Interface mit Extensionmodul an COM1
//          AmpelAuto mit Grn M1, Gelb M2 und Rot M3
//          AmpelFuss mit Grn M5          und Rot M7
//
// --------------------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "AmpelMain.h"

#pragma package(smart_init)
#pragma resource "*.dfm"

// ---------- Modellkonstanten ----------------------------------------------------------

const int mAutoGruen = 1, mAutoGelb = 2, mAutoRot = 3;
const int mFussGruen = 5,                mFussRot = 7;

// ----------- Globale Variable ---------------------------------------------------------

TfrmThreadMain *frmThreadMain;                // --- Globaler Pointer auf Form
TFishFace      *ft;                           // --- Globaler Pointer auf FishFace
                                              //     als Pointer um einheitliche
                                              //     Schreibweise zu erreichen.

// ---------- Globale Events ------------------------------------------------------------

TEvent *FussWunsch = new TEvent(0, 0, 0, ""); // --- Globales Event
                                              //     standard Security, autoReset,
                                              //     not signaled, no Name
TEvent *AutoRot = new TEvent(0, 0, 0, "");
TEvent *FussRot = new TEvent(0, 0, 0, "");


// --------------------- TAmpelAuto -----------------------------------------------------

__fastcall TAmpelAuto::TAmpelAuto(bool CreateSuspended)
        : TThread(CreateSuspended)
{
    // um globale Variable zu vermeiden, knnte man hier auch die Zeiger fr ft und
    // lblStatus bergeben.
    this->Priority = tpNormal;
}

void __fastcall TAmpelAuto::Execute()
{
    do
    {
        // --- Ruhezustand ----------------------------------
        ft->SetMotor(mAutoGruen, ftiEin);
        frmThreadMain->lblStatus->Color = clLime;

        // --- Warten auf Anforderung Fugngerphase --------
        while(!FussWunsch->WaitFor(100) == wrSignaled);

        // --- Schalten Gelb - Rot --------------------------
        ft->SetMotor(mAutoGruen, ftiAus);
        ft->SetMotor(mAutoGelb, ftiEin);
        frmThreadMain->lblStatus->Color = clYellow;
        ft->Pause(1500);
        ft->SetMotor(mAutoGelb, ftiAus);
        ft->SetMotor(mAutoRot, ftiEin);
        frmThreadMain->lblStatus->Color = clRed;

        // --- Signalisieren :  Rot ist geschaltet -------------------
        AutoRot->SetEvent();

        // --- Warten auf Ende der Fugngerphase ----------
        while(!FussRot->WaitFor(100) == wrSignaled);

        // --- Schalten Rot/Gelb - Grn --------------------
        ft->SetMotor(mAutoGelb, ftiEin);
        frmThreadMain->lblStatus->Color = clMaroon;
        ft->Pause(1000);
        ft->SetMotor(mAutoGelb, ftiAus);
        ft->SetMotor(mAutoRot, ftiAus);

    } while(!ft->NotHalt);     // --- Prfen auf Ende des Ampelbetriebs ----
}


// --------------------- TAmpelFuss -----------------------------------------------------

__fastcall TAmpelFuss::TAmpelFuss(bool CreateSuspended)
        : TThread(CreateSuspended)
{
    this->Priority = tpNormal;
}

void __fastcall TAmpelFuss::Execute()
{
    do
    {
        // --- Ruhezustand ---------------------------------
        ft->SetMotor(mFussGruen, ftiAus);
        ft->SetMotor(mFussRot, ftiEin);
        frmThreadMain->lblStatusi->Color = clRed;

        // --- Warten auf RotPhase der AutoAmpel -----------
        while(!AutoRot->WaitFor(100) == wrSignaled);

        // --- Schalten der Grn-Phase ---------------------
        ft->SetMotor(mFussRot, ftiAus);
        ft->SetMotor(mFussGruen, ftiEin);
        frmThreadMain->lblStatusi->Color = clLime;
        ft->Pause(3000);

        // --- Signalisieren : Grnphase ist beendet -------
        FussRot->SetEvent();

    } while(!ft->NotHalt);     // --- Prfen auf Ende des Ampelbetriebes ----
}


// --- ThreadMain : Konstruktor ---------------------------------------------------------

__fastcall TfrmThreadMain::TfrmThreadMain(TComponent* Owner)
    : TForm(Owner) {}

// --- cmdActionClick : Starten des Ampelbetriebes -------------------------------------

void __fastcall TfrmThreadMain::cmdActionClick(TObject *Sender)
{
     ft = new TFishFace(              // --- Instanz von FishFace anlegen
                         false,       //     kein AnalogScan
                         true,        //     mit Slave
                         0);          //     default PollInterval

     try
     {
         ft->OpenInterface("COM1", false);   // Verbindung zum Interface : COM1
     }
     catch(TFishFaceException eft)
     {
        lblStatus->Caption = eft.Text;   // --- wenns nicht klappt : Meldung
        return;                          //     Ende START
     }

     cmdAction->Enabled     = false;     // --- Buttons sortieren
     cmdFussWunsch->Enabled = true;
     cmdEnde->Caption       = "&HALT";


     AmpelAuto = new TAmpelAuto(true);   // --- Threads anlegen
     AmpelFuss = new TAmpelFuss(true);

     AmpelAuto->OnTerminate = AmpelAutoEnde;  // Abschluroutine
     AmpelFuss->OnTerminate = AmpelFussEnde;

     AmpelAuto->Suspended = false;       // --- Starten der Threads
     AmpelFuss->Suspended = false;
}


// --- cmdEndeClick : Beenden des Ampelbetriebes, Programmende bei ENDE ----------------

void __fastcall TfrmThreadMain::cmdEndeClick(TObject *Sender)
{
    if(cmdEnde->Caption == "&HALT")
    {   // --- Beenden des Ampelbetriebes ---------------------------
        lblStatus->Caption = "Beenden wird eingeleitet";
        ft->NotHalt = true;              // --- Schleifenende
        FussWunsch->SetEvent();          //     Warten auf FussPhase beenden
        AmpelAuto->WaitFor();            //     Warten auf Threadende
        AmpelFuss->WaitFor();            //     Warten auf Threadende

        lblStatus->Caption = "--- Finito ---";
        Application->ProcessMessages();
        Sleep(555);

        ft->CloseInterface();            // --- Verbindung zum Interface schlieen

        cmdAction->Enabled = true;       // --- Buttons gerade rcken
        cmdFussWunsch->Enabled = false;
        cmdEnde->Caption = "&ENDE";
    }
    else
    {   // --- Beenden des Programms ------------------------------
        Close();
    }
}

// --- AmpelAutoEnde : EndRoutine des Threads AmpelAuto -------------------------------

void __fastcall TfrmThreadMain::AmpelAutoEnde(TObject * Sender)
{
    lblStatus->Caption = "AmpelAuto : Ende";
    Sleep(500);
}

// --- AmpelFussEnde : EndRoutine des Threads AmpelFuss --------------------------------

void __fastcall TfrmThreadMain::AmpelFussEnde(TObject * Sender)
{
    lblStatusi->Caption = "AmpelFuss : Ende";
    Sleep(500);
}

// --- cmdFussWunschClick : Anmelden Wunsch nach einer Fugngerphase -----------------

void __fastcall TfrmThreadMain::cmdFussWunschClick(TObject *Sender)
{
    FussWunsch->SetEvent();
}

// --- FormCloseQuery : Programmende sperren, wenn noch Ampelbetrieb ----------------

void __fastcall TfrmThreadMain::FormCloseQuery(TObject *Sender,
      bool &CanClose)
{
    if(cmdEnde->Caption == "&HALT") CanClose = false;
}

